@@ -1,9 +1,9 @@
-#' R function for chi-square and G-square test of independence, measures of association, and standardized/moment-corrected
+#' R function for chi-square, (N-1) chi-square, and G-square test of independence, measures of association, and standardized/moment-corrected
 #' standardized/adjusted standardized residuals, visualisation of odds ratio in 2xk tables (where k >= 2)
 #'
-#' @description The function performs the chi-square (and the G-square) test of independence on the input contingency table,
-#' calculates various measures of categorical association, returns standardized, moment-corrected standardized, and adjusted
-#' standardized residuals (with indication of their significance), and calculates relative and absolute contributions to the chi-square.
+#' @description The function performs the chi-square test (both in its original format and in the N-1 version) and the G-square test of independence
+#' on the input contingency table. It also calculates various measures of categorical association, returns standardized, moment-corrected standardized,
+#' and adjusted standardized residuals (with indication of their significance), and calculates relative and absolute contributions to the chi-square.
 #' The p value associated to the chi-square statistic is also calculated on the basis of a permutation-based procedure.
 #' Nicely-formatted output tables are rendered. Optionally, in 2xk tables (where k >= 2), a plot of the odds ratios can be rendered.
 #'
@@ -12,7 +12,7 @@
 #'   \item Phi (only for 2x2 tables)
 #'   \item Phi signed (only for 2x2 tables)
 #'   \item Yule's Q (and p value; only for 2x2 tables)
-#'   \item Odds ratio (with 95perc confidence interval and p value; only for 2x2 tables)
+#'   \item Odds ratio (with 95perc confidence interval, p value, and indication of the magnitude of the effect size; only for 2x2 tables)
 #'   \item Adjusted contingency coefficient C
 #'   \item Cramer's V (with 95perc confidence interval; indication of the magnitude of the effect size according to Cohen is provided for tables with up to 5 degrees of freedom)
 #'   \item Bias-corrected Cramer's V (indication of the magnitude of the effect size according to Cohen is provided for tables with up to 5 degrees of freedom)
@@ -35,6 +35,13 @@
 #' \emph{chisq.stat} is the observed chi-square statistic. For the logic of the calculation, see for example
 #' Baddeley et al. 2016: 387.\cr
 #'
+#' \strong{Chi-square statistics adjusted using the (N-1)/N correction}\cr
+#' The correction is done by dividing the chi-square statistics by (N-1)/N, where N is the table grand total (sample size). The p.value
+#' of the corrected statistic is calculated the regular way. The correction seems particularly relevant for tables where N is smaller than
+#' 20 and where the expected frequencies are equal or larger than 1. The corrected chi-square test proves more conservative when the sample size
+#' is small. As N increases, the term (N-1)/N approaches 1, making the adjusted chi-square value virtually equivalent to the unadjusted value.
+#' See: Upton 1982; Rhoades-Overall1982; Campbel 2007; Richardson 2011. \cr
+#'
 #'
 #' \strong{Moment-corrected standardized residuals}\cr
 #' The moment-corrected standardized residuals are calculated as follows: \cr
@@ -111,6 +118,16 @@
 #' is also applied to the calculation of those pairwise odds ratios (for more information on the plot, see further below).
 #' For the Haldane-Anscombe correction see, for instance, Fleiss-Levin-Paik 2003: 102-103.\cr
 #'
+#' \strong{Odds Ratio effect size magnitude}\cr
+#' The magnitude of the effect indicated by the odds ratio is based on the thresholds (and corresponding reciprocal)
+#' suggested by Chen et al 2010:\cr
+#'
+#'  \itemize{
+#'   \item OR < 1.68 - Very small
+#'   \item 1.68 <= OR < 3.47 - Small
+#'   \item 3.47 <= OR < 6.71 - Medium
+#'   \item OR >= 6.71 - Large
+#' }
 #'
 #' \strong{Odd Ratios plot}\cr
 #' For 2xk table, where k >= 2:\cr
@@ -210,18 +227,20 @@
 #' Also, the function returns a \strong{list containing the following elements}:
 #'  \itemize{
 #'   \item \emph{crosstab}: input contingency table
-#'   \item \emph{exp.freq.}: table of expected frequencies
+#'   \item \emph{exp.freq}: table of expected frequencies
 #'   \item \emph{chisq.values}: cells' chi-square value
-#'   \item \emph{chisq.relat.contrib.}: cells' relative contribution (in percent) to the chi-square statistic
-#'   \item \emph{chisq.abs.contrib.}: cells' absolute contribution (in percent) to the chi-square statistic
+#'   \item \emph{chisq.relat.contrib}: cells' relative contribution (in percent) to the chi-square statistic
+#'   \item \emph{chisq.abs.contrib}: cells' absolute contribution (in percent) to the chi-square statistic
 #'   \item \emph{chisq.statistic}: observed chi-square value
 #'   \item \emph{chisq.p.value}: p value of the chi-square statistic
-#'   \item \emph{chisq.p.value.perm.}: p value based on B permuted tables
+#'   \item \emph{chisq.p.value.perm}: p value based on B permuted tables
+#'   \item \emph{chisq.adj}: chi-square statistic adjusted using the (N-1)/N correction
+#'   \item \emph{chisq.adj.p.value}: p.value of the adjusted chi-square statistic
 #'   \item \emph{Gsq.statistic}: observed G-square value
 #'   \item \emph{Gsq.p.value}: p value of the G-square statistic
 #'   \item \emph{stand.resid.}: table of standardized residuals
-#'   \item \emph{mom.corr.stand.resid.}: table of moment-corrected standardized residuals
-#'   \item \emph{adj.stand.resid.}: table of adjusted standardized residuals
+#'   \item \emph{mom.corr.stand.resid}: table of moment-corrected standardized residuals
+#'   \item \emph{adj.stand.resid}: table of adjusted standardized residuals
 #'   \item \emph{Phi}: Phi coefficient (only for 2x2 tables)
 #'   \item \emph{Phi signed}: signed Phi coefficient (only for 2x2 tables)
 #'   \item \emph{Yule's Q}: Q coefficient (only for 2x2 tables)
@@ -230,6 +249,7 @@
 #'   \item \emph{Odds ratio CI lower boundary}: lower boundary of the 95perc CI
 #'   \item \emph{Odds ratio CI upper boundary}: upper boundary of the 95perc CI
 #'   \item \emph{Odds ratio p.value}: p value of the odds ratio
+#'   \item \emph{Odds ratio effect size}: magnitude of the odds ratio effect size
 #'   \item \emph{Cadj}: adjusted contigency coefficient C
 #'   \item \emph{Cramer's V}: Cramer's V coefficient
 #'   \item \emph{Cramer's V CI lower boundary}: lower boundary of the 95perc CI
@@ -251,7 +271,7 @@
 #' \strong{Note} that the \emph{p values} returned in the above list are expressed in scientific notation, whereas the ones reported in the
 #' output table featuring the tests' result and measures of association are reported as broken down into classes (e.g., <0.05, or <0.01, etc).\cr
 #'
-#' The \strong{following examples}, which use in-built datasets, can be run to fimiliarise with the function:\cr
+#' The \strong{following examples}, which use in-built datasets, can be run to familiarise with the function:\cr
 #'
 #' -perform the test on the in-built 'social_class' dataset\cr
 #' result <- chisquare(social_class)\cr
@@ -282,10 +302,17 @@
 #'
 #' @references Beh E.J., Lombardo R. 2014. Correspondence Analysis: Theory, Practice and New Strategies, Chichester, Wiley.
 #'
-#' @references Beasley TM and Schumacker RE. 1995. Multiple Regression Approach to Analyzing Contingency Tables: Post Hoc and Planned Comparison Procedures. The Journal of Experimental Education, 64(1).
+#' @references Beasley TM and Schumacker RE. 1995. Multiple Regression Approach to Analyzing Contingency Tables: Post Hoc and Planned Comparison Procedures.
+#' The Journal of Experimental Education, 64(1).
 #'
 #' @references Bergsma, W. 2013. A bias correction for Cramér's V and Tschuprow's T. Journal of the Korean Statistical Society. 42 (3).
 #'
+#'  @references Campbell, I. (2007). Chi-squared and Fisher–Irwin tests of two-by-two tables with small sample recommendations.
+#' In Statistics in Medicine (Vol. 26, Issue 19, pp. 3661–3675).
+#'
+#' @references Chen, H., Cohen, P., and Chen, S. (2010). How Big is a Big Odds Ratio? Interpreting the Magnitudes of Odds Ratios in Epidemiological Studies.
+#' In Communications in Statistics - Simulation and Computation (Vol. 39, Issue 4, pp. 860–864).
+#'
 #' @references Cohen, J. 1988. Statistical power analysis for the behavioral sciences (2nd ed). Hillsdale, N.J: L. Erlbaum Associates.
 #'
 #' @references Fleiss, J. L., Levin, B., & Paik, M. C. 2003. Statistical Methods for Rates and Proportions (3rd ed.). Wiley.
@@ -294,10 +321,18 @@
 #'
 #' @references Reynolds, H. T. 1984. Analysis of Nominal Data (Quantitative Applications in the Social Sciences) (1st ed.). SAGE Publications.
 #'
+#' @references Rhoades, H. M., & Overall, J. E. (1982). A sample size correction for Pearson chi-square in 2×2 contingency tables. In Psychological Bulletin (Vol. 91, Issue 2, pp. 418–423).
+#' American Psychological Association (APA).
+#'
+#' @references Richardson, J. T. E. (2011). The analysis of 2 × 2 contingency tables-Yet again. In Statistics in Medicine (Vol. 30, Issue 8, pp. 890–890).
+#'
 #' @references Sheskin, D. J. 2011. Handbook of Parametric and Nonparametric Statistical Procedures, Fifth Edition (5th ed.). Chapman and Hall/CRC.
 #'
 #' @references Smithson M.J. 2003. Confidence Intervals, Quantitative Applications in the Social Sciences Series, No. 140. Thousand Oaks, CA: Sage.
 #'
+#' @references Upton, G. J. G. (1982). A Comparison of Alternative Tests for the 2 × 2 Comparative Trial. In Journal of the Royal Statistical Society.
+#' Series A (General) (Vol. 145, Issue 1, p. 86).
+#'
 #'
 #' @export
 #'
@@ -311,6 +346,10 @@
 #' result <- chisquare(social_class, B=99)
 #'
 #'
+#' # Perform the test on a 2x2 subset
+#' result <- chisquare(social_class[c(1:2), c(1:2)], B=99)
+#'
+#'
 #'
 chisquare <- function(data, B=999, plot.or= FALSE, reference.level = 1, row.level = 1, or.alpha = 0.05, adj.alpha=FALSE, format="short", graph=FALSE, tfs=14){
   VALUE <- NULL
@@ -377,19 +416,27 @@ chisquare <- function(data, B=999, plot.or= FALSE, reference.level = 1, row.leve
   }
 
   #put the above function to work and extract the chi-sq statistic
-  #the chi-sq values, and the expected frequencies to be used later on
+  #the chi-sq values, and the expected frequencies, and the adjusted chi-sq to be used later on
   chisq.stat <- calc(df)$chisq.stat
   chisq.values <- round(calc(df)$chisq.values,3)
   exp.freq <- round(calc(df)$exp.freq,3)
+  chisq.adj <- round(chisq.stat * ((n-1)/n), 3)
 
-  #p value
+
+  #p values
   p <- as.numeric(format(pchisq(q=chisq.stat, df=degrees.of.f, lower.tail=FALSE), scientific = T))
+  p.chisq.adj <- as.numeric(format(pchisq(q=chisq.adj, df=degrees.of.f, lower.tail=FALSE), scientific = T))
 
   p.to.report <- ifelse(p < 0.001, "< 0.001",
                         ifelse(p < 0.01, "< 0.01",
                                ifelse(p < 0.05, "< 0.05",
                                       round(p, 3))))
 
+  p.chisq.adj.to.report <- ifelse(p.chisq.adj < 0.001, "< 0.001",
+                                  ifelse(p.chisq.adj < 0.01, "< 0.01",
+                                         ifelse(p.chisq.adj < 0.05, "< 0.05",
+                                                round(p.chisq.adj, 3))))
+
   #calculate the chi-sq statistic B times, using the above-defined function
   extract <- function(x)  calc(x)$chisq.stat
   chistat.perm <- sapply(r2dtable(B, sr, sc), extract)
@@ -691,6 +738,35 @@ chisquare <- function(data, B=999, plot.or= FALSE, reference.level = 1, row.leve
     or.p.value <- "-"
   }
 
+  #Defice a function to calculate the effect-size for the odds ratio
+  #based on the thresholds suggested by Chen et al 2010.
+  effect_size_from_OR <- function(OR) {
+
+    # If the OR is less than 1, consider its reciprocal for categorization
+    if (OR < 1) {
+      OR <- 1 / OR
+    }
+
+    # Check the OR value
+    if (OR < 1.68) {
+      return("Very small")
+    } else if (OR >= 1.68 && OR < 3.47) {
+      return("Small")
+    } else if (OR >= 3.47 && OR < 6.71) {
+      return("Medium")
+    } else {
+      return("Large")
+    }
+  }
+
+  #Put the above function to work and work out the
+  #odds ratio's effect size
+  if (or == "-") {
+    or.effect.size <- "-"
+  } else {
+    or.effect.size <- effect_size_from_OR(or)
+  }
+
   #Cohen's k
   sum.observed.diag <- sum(diag(as.matrix(df)))
   sum.expected.diag <- sum(diag(as.matrix(exp.freq)))
@@ -750,11 +826,13 @@ chisquare <- function(data, B=999, plot.or= FALSE, reference.level = 1, row.leve
 
   #define a vector for the labels of the statistics to be returned
   statistics <- c("Chi-square",
+                  "Chi-square adjusted",
                   "G-square",
                   "Phi",
                   "Phi signed",
                   "Yule's Q",
                   "Odds ratio",
+                  "Odds ratio effect size",
                   "Cadj",
                   paste0("Cramer's V", V.effect.size.to.report),
                   paste0("Cramer's V bias-corrected", Vbc.effect.size.to.report),
@@ -769,11 +847,13 @@ chisquare <- function(data, B=999, plot.or= FALSE, reference.level = 1, row.leve
 
   #define a vector for the statistics to be returned
   values.to.report <-c(paste0(chisq.stat, " (df: ", degrees.of.f, "; p: ", p.to.report, "; perm. p: ", p.uppertail.to.report, ")"),
+                       paste0(chisq.adj, " (p: ", p.chisq.adj.to.report, ")"),
                        paste0(Gsquared, " (p: ", p.Gsquared.to.report, ")"),
                        phi,
                        phi.signed,
                        report.of.Q,
                        report.of.or,
+                       or.effect.size,
                        Cadj,
                        report.of.V,
                        V.bc,
@@ -811,8 +891,9 @@ chisquare <- function(data, B=999, plot.or= FALSE, reference.level = 1, row.leve
   input.table.out <- gt::tab_source_note(input.table.out,
                                          md(paste0("*Chi-square: ", chisq.stat,
                                                    " (df: ", degrees.of.f, "; p: ", p.to.report,") <br>
-                                    G-square: ", Gsquared," (p: ", p.Gsquared.to.report, ") <br>
-                                    Cramer's V: ", report.of.V,
+                                                   Chi-square adj: ", chisq.adj, "(p: ", p.chisq.adj.to.report, ") <br>
+                                                   G-square: ", Gsquared," (p: ", p.Gsquared.to.report, ") <br>
+                                                   Cramer's V: ", report.of.V,
                                                    "<br> Goodman-Kruskal's lambda (symmetric): ", lambda,
                                                    "<br> Goodman-Kruskal's gamma: ", gamma.coeff,
                                                    "<br> Number of cells with a significant standardized residual: ", sum(abs(stand.res) > z.crit.two.tailed),
@@ -993,18 +1074,20 @@ chisquare <- function(data, B=999, plot.or= FALSE, reference.level = 1, row.leve
 
   #create a list of results to be returned
   results <- list("crosstab"=df,
-                  "exp.freq."=exp.freq,
+                  "exp.freq"=exp.freq,
                   "chisq.values"=chisq.values,
-                  "chisq.relat.contrib."=relative.contrib,
-                  "chisq.abs.contrib."=absolute.contrib,
+                  "chisq.relat.contrib"=relative.contrib,
+                  "chisq.abs.contrib"=absolute.contrib,
                   "chisq.statistic"=chisq.stat,
                   "chisq.p.value"=p,
-                  "chisq.p.value.perm."=p.uppertail,
+                  "chisq.p.value.perm"=p.uppertail,
+                  "chisq.adj"=chisq.adj,
+                  "chisq-adj.p.value"=p.chisq.adj,
                   "Gsq.statistic"=Gsquared,
                   "Gsq.p.value"=p.Gsquared,
-                  "stand.resid."=stand.res,
-                  "mom.corr.stand.resid."=mom.corr.stand.res,
-                  "adj.stand.resid."=adj.stand.res,
+                  "stand.resid"=stand.res,
+                  "mom.corr.stand.resid"=mom.corr.stand.res,
+                  "adj.stand.resid"=adj.stand.res,
                   "Phi"=phi,
                   "Phi.signed"=phi.signed,
                   "Yule's Q"=Q,
@@ -1013,6 +1096,7 @@ chisquare <- function(data, B=999, plot.or= FALSE, reference.level = 1, row.leve
                   "Odds ratio CI lower boundary"=or.lower.ci,
                   "Odds ratio CI upper boundary"=or.upper.ci,
                   "Odds ratio p.value"=or.p.value,
+                  "Odds ratio effect size"=or.effect.size,
                   "Cadj"=Cadj,
                   "Cramer's V"=V,
                   "Cramer's V CI lower boundary"=V.lower,
